#!/usr/bin/env python

import argparse
import os
import numpy as np
import iio
import scipy.misc
import errno
import scipy.linalg
from scipy.stats import binom

np.warnings.filterwarnings('ignore')

# Joint PCA projection for the three datas
def PCA_upper(data_1, data_2, data_3, data_ref):
    m, n = data_ref.shape
    # Use high precision
    data_ref = np.asarray(data_ref, dtype=np.float64)
    # Remove 1st order statistics
    data_1 -= data_ref.mean(axis=0)
    data_2 -= data_ref.mean(axis=0)
    data_3 -= data_ref.mean(axis=0)
    data_ref -= data_ref.mean(axis=0)
    C = np.cov(data_ref, rowvar=False)
    D, eigs = scipy.linalg.eigh(C)
    # Remove low precision eigenvalues
    # D is in ascending order.
    D[D < 1e-15 * D[-1]] = 0
    # PCA: C^-1/2 = D^-1/2 E' D:values, E = Vectors
    # ZCA: C^-1/2 = E D^-1/2 E'
    num_channels = min(5, sum(D>0))
    C_m_1_2 = np.zeros([num_channels, n])
    for c0 in range(num_channels):
        c = c0 + (n - num_channels)
        assert (D[c] > 0)
        C_m_1_2[c0,:] = (1./np.sqrt(D[c])) * eigs[:,c]
    return np.dot(C_m_1_2, data_1.T).T, np.dot(C_m_1_2, data_2.T).T, np.dot(C_m_1_2, data_3.T).T, np.dot(C_m_1_2, data_ref.T).T

def pca_on_img(data_1, data_2, data_3, data_ref):
    [n, m, c] = data_ref.shape
    [nc, mc, _] = data_1.shape
    if (n*m < c):
        # cannot do PCA
        return data_1, data_2, data_3
    data_ref = data_ref.reshape([n * m, c])
    data_1 = data_1.reshape([nc * mc, c])
    data_2 = data_2.reshape([nc * mc, c])
    data_3 = data_3.reshape([nc * mc, c])
    pca_1, pca_2, pca_3, _ = PCA_upper(data_1, data_2, data_3, data_ref)
    new_c = pca_1.shape[1]
    pca_1 = pca_1.reshape([nc, mc, new_c])
    pca_2 = pca_2.reshape([nc, mc, new_c])
    pca_3 = pca_3.reshape([nc, mc, new_c])
    return pca_1, pca_2, pca_3

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('pre', type=str, help=('path to the pre processed file (tiff)'))
    parser.add_argument('model', type=str, help=('path to model file (tiff)'))
    parser.add_argument('res', type=str, help=('path to residual file (tiff)'))
    parser.add_argument('c1', type=int, help=('index for the C1 channels'))
    parser.add_argument('c2', type=int, help=('index for the C2 channels'))
    args = parser.parse_args()

    pre = iio.read(args.pre)
    model = iio.read(args.model)
    res = iio.read(args.res)

    pre_c1 = pre[:,:,:args.c1]
    model_c1 = model[:,:,:args.c1]
    res_c1 = res[:,:,:args.c1]

    pre_c2 = pre[:,:,args.c2:]
    model_c2 = model[:,:,args.c2:]
    res_c2 = res[:,:,args.c2:]

    # Process C1 results
    if pre_c1.shape[2] == 1:
        mini = np.percentile(pre_c1, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(pre_c1, 99, axis=(0,1), keepdims=True)

        pre_c1 = (pre_c1-mini)/(maxi-mini)*255
        model_c1 = (model_c1-mini)/(maxi-mini)*255

        mini = np.percentile(res_c1, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(res_c1, 99, axis=(0,1), keepdims=True)
        res_c1 = (res_c1-mini)/(maxi-mini)*255

        iio.write("pre_c1.png", pre_c1)
        iio.write("model_c1.png", model_c1)
        iio.write("res_c1.png", res_c1)
    else:
        pre_c1, model_c1, res_c1 = pca_on_img(pre_c1, model_c1, res_c1, pre_c1)

        mini = np.percentile(pre_c1, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(pre_c1, 99, axis=(0,1), keepdims=True)

        pre_c1 = (pre_c1-mini)/(maxi-mini)*255
        model_c1 = (model_c1-mini)/(maxi-mini)*255

        mini = np.percentile(res_c1, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(res_c1, 99, axis=(0,1), keepdims=True)
        res_c1 = (res_c1-mini)/(maxi-mini)*255

        if pre_c1.shape[2] == 2:
            epre_c1 = np.zeros((pre.shape[0], pre.shape[1], 3))
            emodel_c1 = np.zeros((pre.shape[0], pre.shape[1], 3))
            eres_c1 = np.zeros((pre.shape[0], pre.shape[1], 3))
            
            epre_c1[:,:,:2] = pre_c1
            emodel_c1[:,:,:2] = model_c1
            eres_c1[:,:,:2] = res_c1


            iio.write("pre_c1.png", epre_c1)
            iio.write("model_c1.png", emodel_c1)
            iio.write("res_c1.png", eres_c1)
        else:
            iio.write("pre_c1.png", pre_c1[:,:,:3])
            iio.write("model_c1.png", model_c1[:,:,:3])
            iio.write("res_c1.png", res_c1[:,:,:3])

    # Process C2 results
    if pre_c2.shape[2] == 1:
        mini = np.percentile(pre_c2, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(pre_c2, 99, axis=(0,1), keepdims=True)

        pre_c2 = (pre_c2-mini)/(maxi-mini)*255
        model_c2 = (model_c2-mini)/(maxi-mini)*255

        mini = np.percentile(res_c2, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(res_c2, 99, axis=(0,1), keepdims=True)
        res_c2 = (res_c2-mini)/(maxi-mini)*255

        iio.write("pre_c2.png", pre_c2)
        iio.write("model_c2.png", model_c2)
        iio.write("res_c2.png", res_c2)
    else:
        pre_c2, model_c2, res_c2 = pca_on_img(pre_c2, model_c2, res_c2, pre_c2)

        mini = np.percentile(pre_c2, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(pre_c2, 99, axis=(0,1), keepdims=True)

        pre_c2 = (pre_c2-mini)/(maxi-mini)*255
        model_c2 = (model_c2-mini)/(maxi-mini)*255

        mini = np.percentile(res_c2, 5, axis=(0,1), keepdims=True)
        maxi = np.percentile(res_c2, 99, axis=(0,1), keepdims=True)
        res_c2 = (res_c2-mini)/(maxi-mini)*255

        if pre_c2.shape[2] == 2:
            epre_c2 = np.zeros((pre.shape[0], pre.shape[1], 3))
            emodel_c2 = np.zeros((pre.shape[0], pre.shape[1], 3))
            eres_c2 = np.zeros((pre.shape[0], pre.shape[1], 3))
            
            epre_c2[:,:,:2] = pre_c2
            emodel_c2[:,:,:2] = model_c2
            eres_c2[:,:,:2] = res_c2


            iio.write("pre_c2.png", epre_c2)
            iio.write("model_c2.png", emodel_c2)
            iio.write("res_c2.png", eres_c2)
        else:
            iio.write("pre_c2.png", pre_c2[:,:,:3])
            iio.write("model_c2.png", model_c2[:,:,:3])
            iio.write("res_c2.png", res_c2[:,:,:3])
