/*
 * Copyright (c) 2017, Thibaud Ehret <ehret.thibaud@gmail.com>
 * All rights reserved.
 *
 * This program is free software: you can use, modify and/or
 * redistribute it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later
 * version. You should have received a copy of this license along
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * @file 
 * @brief 
 *
 * @author Thibaud Ehret <ehret.thibaud@gmail.com>
 **/

#include "model.h"
#include "../Utilities/comparators.h"

extern"C"
{
void nnls_(double *a, int *mda, int *m, int *n, double *b, double *x, double *rnorm, double *w, double *zz, int *index, int *mode);
}
#include "ls.hpp"

#define SQRT2_INV 0.7071067811865475
#define NTHREAD 32

/**
 * @brief Compute the final weighted aggregation.
 *
 * result: contain the aggregation, will contain the result at the end.
 * weight: contains the weights used during the aggregation;
 *
 * @return : none.
 **/
inline void computeWeightedAggregation(
	std::vector<float>& result
,	const ImageSize imSize
,	std::vector<float> const& weight
){
	for (unsigned y = 0; y < imSize.height; ++y)
	for (unsigned x = 0; x < imSize.width; ++x)
	for (unsigned c = 0; c < imSize.nChannels; ++c)
        result[c + x*imSize.nChannels + y*imSize.nChannels*imSize.width] /= weight[x + y*imSize.width];
}

void projectOnCone(
	double* patch
,	double* subspace
,   double* x
,	const unsigned p_nbVects
,   const unsigned dim
){
	for(int i = 0; i < p_nbVects; ++i)
		x[i] = 0.;

    // FIXME possible speedup here?
	double normRes;
	int mode;
	int* index = new int[p_nbVects];
	for(int i = 0; i < p_nbVects; ++i)
		index[i] = 0;
	double* w = new double[p_nbVects];
	for(int i = 0; i < p_nbVects; ++i)
		w[i] = 0.;
	double* zz = new double[dim];
	for(int i = 0; i < dim; ++i)
		zz[i] = 0.;
	int p_nbVects_ = p_nbVects;
	int dim_ = dim;
	double* empty = new double[p_nbVects];
	for(int i = 0; i < p_nbVects; ++i)
		empty[i] = 0.;

    nnls_(subspace, &dim_, &dim_, &p_nbVects_, patch, x, &normRes, w, zz, index, &mode);

    if(mode != 1)
        printf("Computation ended with error %d\n", mode);

	delete[] index;
	delete[] w;
	delete[] zz;
	delete[] empty;
}

void projectOnSubspace(
	double* patch
,	double* subspace
,   double* x
,	const unsigned p_nbVects
,   const unsigned dim
,   const unsigned dimFull
){
	// Compute an orthonormal basis
	float* orthnorm = new float[p_nbVects*dimFull];
	std::vector<std::vector<float> > weights(p_nbVects, std::vector<float>(p_nbVects, 0.f));
	float temp;

	for(unsigned k = 0; k < p_nbVects; ++k) 
	{
		for(unsigned j = 0; j < dimFull; ++j)
		{
			orthnorm[k + j * p_nbVects] = subspace[k + j * p_nbVects];
		}
		for(unsigned l = 0; l < k; ++l)
		{
			temp = 0.;
			for(unsigned j = 0; j < dim; ++j)
			{
				temp += orthnorm[l + j * p_nbVects] * orthnorm[k + j * p_nbVects];
			}
			weights[k][l] = temp;
			for(unsigned j = 0; j < dimFull; ++j)
			{
				orthnorm[k + j * p_nbVects] -= temp * orthnorm[l + j * p_nbVects];
			}
		}

		temp = 0.;
		for(unsigned j = 0; j < dim; ++j)
		{
			temp += orthnorm[k + j * p_nbVects] * orthnorm[k + j * p_nbVects];
		}
		temp = std::sqrt(temp);
		if(temp > 0.000001)
		{
			for(unsigned j = 0; j < dimFull; ++j)
			{
				orthnorm[k + j * p_nbVects] = orthnorm[k + j * p_nbVects] / temp;
			}
		}
	}

	// Compute the projection coefficients (scalar product) in x
    for(unsigned k = 0; k < p_nbVects; ++k) 
    {
        x[k] = 0.;
        for(unsigned j = 0; j < dim; ++j)
        {
            x[k] += patch[j] * orthnorm[k + j * p_nbVects];
        }
    }
    for(int i = 0; i < dimFull; ++i)
        patch[i] = 0;
    for(unsigned j = 0; j < dimFull; ++j)
    for(unsigned k = 0; k < p_nbVects; ++k) 
    {
        patch[j] += x[k] * orthnorm[k + j * p_nbVects];
    }

}


void computeModel(
	const std::vector<float>& im
,	ImageSize imSize
,	const std::vector<float>& ref
,	ImageSize refSize
,	std::vector<float> &model
,   int maxDim
,	Params const& params
){
	//! Parameters initialization
	const unsigned sPx = params.sizePatch;

	//! Weight sum per pixel
	std::vector<float> weight(imSize.width*imSize.height, 0.f);

	const unsigned patch_num = params.nSimilarPatches;
	std::vector<std::pair<float, unsigned> > index(patch_num);

    std::vector<double> patch(imSize.nChannels*sPx*sPx);
    std::vector<double> subspace(imSize.nChannels*sPx*sPx * params.nSimilarPatches);
    std::vector<double> coeffs(params.nSimilarPatches);

	//! Create the VPtree forest
	ImagePatchManager pm(ref, refSize, sPx, maxDim);
	std::vector<PartitionTree*> trees; 
	PrmTree paramTree;
	paramTree.epsilon = 0.; 
	paramTree.nbVP = 1;
	paramTree.subS = 1000;
	paramTree.rand = 5;
    paramTree.prms = params;
#ifdef _OPENMP
#pragma omp parallel for schedule(dynamic) \
	shared(trees) 
#endif
	for(int i = 0; i < params.nbTrees; ++i)
		trees.push_back(new VPTree(pm, paramTree));	

	ForestManager fm(pm, trees, params.nSimilarPatches, params);

	DatabasePatchManager dbpm(ref, refSize, sPx, maxDim);
	dbpm.setCurrentImage(im, imSize);
    fm.updatePM(dbpm);

	LocalRefinement lpp(dbpm, fm, patch_num, params);

    int X = 199, Y = 175;

	// Iterate on patches this way so there is no overlap during the aggregation. This allow to compute all patches in parallel.
	for (unsigned shifty = 0; shifty < sPx; ++shifty)
	for (unsigned shiftx = 0; shiftx < sPx; ++shiftx)
    {
#ifdef _OPENMP
#pragma omp parallel for num_threads(NTHREAD) schedule(dynamic) \
        shared(lpp, model, weight) \
        firstprivate(index, patch, subspace, coeffs)
#endif

        for (unsigned py = shifty; py < imSize.height - sPx + 1; py+=sPx)
            for (unsigned px = shiftx; px < imSize.width - sPx + 1; px+=sPx)
            {
                const unsigned ij  = px + imSize.width*py;
                const unsigned ij3 = (px + imSize.width*py)*imSize.nChannels;

                //! Search for similar patches around the reference one
                unsigned nSimP = lpp.estimateSimilarPatches(index, ij3, params.same);


                if(params.mode == 1)
                {
                    // LS mode
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    for (unsigned c = 0; c < maxDim; c++)
                        patch[c*sPx*sPx + x + y*sPx] = ref[ij3 + y*imSize.width*imSize.nChannels + x*imSize.nChannels + c];

                    // Load the nearest neighbors
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    for (unsigned c = 0; c < maxDim; c++)
                    for (unsigned k = 0; k < nSimP; ++k)
                        //subspace[c*sPx*sPx*nSimP + x*nSimP + y*sPx*nSimP + k] = 
                        subspace[c*sPx*sPx + x + y + k*sPx*sPx*maxDim] = 
                            ref[index[k].second  + y*refSize.width*refSize.nChannels + x*refSize.nChannels + c];

                    // Project
                    //projectOnSubspace(patch.data(), subspace.data(), coeffs.data(), nSimP, sPx*sPx*maxDim, sPx*sPx*imSize.nChannels);
                    //projectOnSubspace(patch.data(), subspace.data(), coeffs.data(), nSimP, sPx*sPx*imSize.nChannels, sPx*sPx*imSize.nChannels);
                    double* co = svd_solve(sPx*sPx*maxDim, nSimP, subspace.data(), patch.data());

                    // Aggregate back
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    {
                        for (unsigned c = 0; c < imSize.nChannels; c++)
                        for (unsigned k = 0; k < nSimP; ++k)
                        {
                            model[ij3 + y*imSize.width*imSize.nChannels + x*imSize.nChannels + c] += co[k] * ref[index[k].second  + y*refSize.width*refSize.nChannels + x*refSize.nChannels + c];
                        }

                        weight[ij + y*imSize.width + x] += 1;
                    }
                    free(co);
                }
                if(params.mode == 2)
                {
                    // NNLS mode
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    for (unsigned c = 0; c < maxDim; c++)
                        patch[c + x*maxDim + y*maxDim*sPx] = ref[ij3 + y*imSize.width*imSize.nChannels + x*imSize.nChannels + c];

                    // Load the nearest neighbors
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    for (unsigned c = 0; c < maxDim; c++)
                    for (unsigned k = 0; k < nSimP; ++k)
                        subspace[c + x*maxDim + y*maxDim*sPx + k*maxDim*sPx*sPx] = 
                            ref[index[k].second  + y*refSize.width*refSize.nChannels + x*refSize.nChannels + c];

                    // Project
                    projectOnCone(patch.data(), subspace.data(), coeffs.data(), nSimP, sPx*sPx*maxDim);

                    // Aggregate back
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    {
                        for (unsigned c = 0; c < imSize.nChannels; c++)
                        for (unsigned k = 0; k < nSimP; ++k)
                        {
                            model[ij3 + y*imSize.width*imSize.nChannels + x*imSize.nChannels + c] += coeffs[k] * ref[index[k].second  + y*refSize.width*refSize.nChannels + x*refSize.nChannels + c];
                        }

                        weight[ij + y*imSize.width + x] += 1;
                    }
                }
                else
                {
                    // NLM mode
                    // Denormalize the distances
                    for(unsigned k = 0; k < nSimP; ++k)
                    {
                        index[k].first *= 255.f;
                        index[k].first *= index[k].first;
                    }

                    float norm_factor = 0.f;
                    float h2 = params.h*params.h;
                    for (unsigned k = 0; k < nSimP; ++k)
                    {
                        index[k].first = exp(-index[k].first/h2);
                        norm_factor += index[k].first;
                    }
                    if(norm_factor > 0)
                        norm_factor = 1.f / norm_factor;
                    else
                    {
                        for(int i = 0; i < nSimP; ++i)
                            index[i].first = 1;
                        norm_factor = 1.f / nSimP;
                    }

                    // Compute the average patch while aggregating it.
                    for (unsigned y = 0; y < sPx; y++)
                    for (unsigned x = 0; x < sPx; x++)
                    {
                        for (unsigned c = 0; c < imSize.nChannels; c++)
                        {
                            for (unsigned k = 0; k < nSimP; ++k)
                                model[ij3 + y*imSize.width*imSize.nChannels + x*imSize.nChannels + c] +=
                                    index[k].first * ref[index[k].second  + y*refSize.width*refSize.nChannels + x*refSize.nChannels + c] * norm_factor;
                        }

                        weight[ij + y*imSize.width + x] += 1;
                    }
                }
            }
    }

	//! Weighted aggregation
	computeWeightedAggregation(model, imSize, weight);

	//! Delete trees
	for(int i = 0; i < params.nbTrees; ++i)
		delete trees[i];
}
